// Sun, 18 Jun 2000
// Y.Takagi

#include <windows.h>
#include <iostream>
#include "IppContent.h"
#include "IppURLConnection.h"

using namespace std;

int main(int argc, char **argv)
{
	if (argc < 2) {
		cout << "Usage:" << '\n';
		cout << '\n';
		cout << "print file:" << '\n';
		cout << "testipp [URL] [Filename]" << '\n';
		cout << '\n';
		cout << "get printer attributes:" << '\n';
		cout << "testipp [URL]" << endl;
		return 0;
	}

	char *printerUrl = *++argv;
	char *file = NULL;
	bool print_file;

	if (argc == 2) {
		print_file = false;
	} else {
		file = *++argv;
		print_file = true;
	}

	char username[256];
	DWORD size = sizeof(username) / sizeof(username[0]);
	if (!GetUserName(username, &size)) {
		strcpy(username, "baron");
	}

	IppContent *request = new IppContent;
	if (print_file) {
		request->setOperationId(IPP_PRINT_JOB);
		request->setDelimiter(IPP_OPERATION_ATTRIBUTES_TAG);
		request->setCharset("attributes-charset", "utf-8");
		request->setNaturalLanguage("attributes-natural-language", "en-us");
		request->setURI("printer-uri", printerUrl);
		request->setMimeMediaType("document-format", "application/octet-stream");
		request->setNameWithoutLanguage("requesting-user-name", username);
//		request->setNameWithoutLanguage("job-name", file);	// optional
		request->setDelimiter(IPP_END_OF_ATTRIBUTES_TAG);
		request->setRawData(file);
	} else {
		request->setOperationId(IPP_GET_PRINTER_ATTRIBUTES);
		request->setDelimiter(IPP_OPERATION_ATTRIBUTES_TAG);
		request->setCharset("attributes-charset", "utf-8");
		request->setNaturalLanguage("attributes-natural-language", "en-us");
		request->setNameWithoutLanguage("requesting-user-name", username);
		request->setURI("printer-uri", printerUrl);
		request->setDelimiter(IPP_END_OF_ATTRIBUTES_TAG);
	}

	URL url(printerUrl);
	IppURLConnection conn(url);
	conn.setIppRequest(request);
	conn.setRequestProperty("Connection", "close");


	HTTP_RESPONSECODE response_code = conn.getResponseCode();
	if (response_code != HTTP_OK) {
		if (response_code != HTTP_UNKNOWN) {
			cout << conn.getResponseMessage() << '\n';
		} else {
			cout << "cannot connect to IPP server." << '\n';
		}
	} else {
		const IppContent *ipp_response = conn.getIppResponse();
		if (ipp_response->fail()) {
			cout << ipp_response->getStatusMessage() << '\n';
		} else {
			cout << "Success." << '\n';
		}
	}

	const char *contentType = conn.getContentType();
	if (contentType) {
		cout << "ContentType: " << contentType << endl;
	}
	conn.printIppResponse(cout);

	return 0;
}
