// Sun, 18 Jun 2000
// Y.Takagi

#include <windows.h>
#include <winsock.h>

#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <string>

#include "LpsClient.h"

using namespace std;

bool print(
	const char *server,
	const char *printer,
	const char *filename)
{
	ifstream ifs(filename, ios::binary);

	if (!ifs) {
		cerr << "couldn't open file." << endl;
		return false;
	}

	char hostname[128];
	gethostname(hostname, sizeof(hostname));

	char username[128];
#ifdef WIN32
	unsigned long length = sizeof(hostname);
	GetUserName(username, &length);
#else
	getusername(username, sizeof(username));
#endif

	int jobid = 1;

	ostringstream cfname;
	cfname << "cfA" << setw(3) << setfill('0') << jobid << hostname;

	ostringstream dfname;
	dfname << "dfA" << setw(3) << setfill('0') << jobid << hostname;

	ostringstream cf;
	cf << 'H' << hostname     << '\n';
	cf << 'P' << username     << '\n';
	cf << 'l' << dfname.str() << '\n';
	cf << 'U' << dfname.str() << '\n';

	long cfsize = cf.str().length();

	ifs.seekg(0, ios::end);
	long dfsize = ifs.tellg();
	ifs.seekg(0, ios::beg);

	try {

		LpsClient lpr(server);

		lpr.connect();
		lpr.receiveJob(printer);

		lpr.receiveControlFile(cfsize, cfname.str().c_str());
		lpr.transferData(cf.str().c_str(), cfsize);
		lpr.endTransfer();

		lpr.receiveDataFile(dfsize, dfname.str().c_str());
		lpr.transferData(ifs, dfsize);
		lpr.endTransfer();
	}

	catch (LPSException &err) {
		cerr << err.what() << endl;
		return false;
	}

	return true;
}

int main(int argc, char **argv)
{
	if (argc < 4) {
		return -1;
	}

	const char *server   = *++argv;
	const char *printer  = *++argv;
	const char *filename = *++argv;

	return print(server, printer, filename) ? 0 : -1;
}
